/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using Gtk;
using System.Collections.Generic;

namespace Sagittarius
{
	/// <summary>
	/// Reads the directories the user want to a TreeStore
	/// </summary>
	public class FileSystemReader
	{
		private TreeStore dirTree;
		private bool showHidden;
		/// <summary>
		/// Reads the '/' dir to the TreeStore
		/// </summary>
		/// <param name="showHidden">
		/// If set, hidden directories are also listed <see cref="System.Boolean"/>
		/// </param>
		public FileSystemReader(bool showHidden)
		{
			//Console.WriteLine("Entering FsReader constructor(bool)");
			this.showHidden=showHidden;
			dirTree = new TreeStore(typeof (DirInfo));
			DirInfo root = new DirInfo("/");
			TreeIter rootIter = dirTree.AppendValues(root); 
			dirTree.AppendValues(rootIter,new DirInfo()); //make a dummy DirInfo under it for the TreeView to display expanders near its icon
			//ReadDir(root, rootIter);
			
			DirInfo home = new DirInfo(System.Environment.GetEnvironmentVariable("HOME"));
			TreeIter homeIter = dirTree.AppendValues(home);
			dirTree.AppendValues(homeIter,new DirInfo()); //make a dummy DirInfo under it for the TreeView to display expanders near its icon
			//ReadDir(home, homeIter);
			//Console.WriteLine("Leaving FsReader constructor(bool)");
		}

		public FileSystemReader(bool showHidden, List<string> directories)
		{
			//Console.WriteLine("Entering FsReader constructor(bool, List)");
			this.showHidden=showHidden;
			dirTree = new TreeStore(typeof (DirInfo));
			foreach(string directory in directories)
			{
				DirInfo root = new DirInfo(directory);
				TreeIter iter = dirTree.AppendValues(root); 
				dirTree.AppendValues(iter,new DirInfo()); //make a dummy DirInfo under it for the TreeView to display expanders near its icon
				//ReadDir(root, iter);
			}
			//Console.WriteLine("Leaving FsReader constructor(bool, List)");
		}

		/// <summary>
		/// Reads the subdirectories of a directory, and appends the 
		/// corresponding DirInfo objects to the TreeStore
		/// </summary>
		/// <param name="parent">
		/// DirInfo, the parent directory<see cref="DirInfo"/>
		/// </param>
		/// <param name="iter">
		/// TreeIter, set to the parent DirInfo <see cref="Gtk.TreeIter"/>
		/// </param>
		private void ReadDir(DirInfo parent, Gtk.TreeIter iter)
		{
			//Console.WriteLine("Entering FSReader.ReadDir("+parent.FullName+")");
			if(parent.Explorable) //if the directory is readable/executable for the user
			{
				//Console.WriteLine(" "+parent.FullName+" is readable");
				//Console.WriteLine("  "+parent.FullName+" wasn't explored before. Exploring...");
				foreach (DirInfo child in parent.GetDirectories()) // with all its subdirectories
				{
					//Console.WriteLine("   "+ "Subdirectory: "+child.FullName);
					if(showHidden) 
					{
						//Console.WriteLine("    "+"Appending the directory "+child.FullName+" to the listStore");
					
						TreeIter childIter = dirTree.AppendValues(iter, child); // append the directory to the TreeStore
						if (child.hasSubDirectories) // if it has subdirectories
						{
							//Console.WriteLine("     "+"Directory "+child.FullName+" has subdirectories, making a dummy Dirinfo, for the little triangle");
							dirTree.AppendValues(childIter,new DirInfo()); //make a dummy DirInfo under it for the TreeView to display expanders near its icon
						}
					}
					else if (!(child.Name[0]=='.'))
					{
						//Console.WriteLine("    "+"Appending the directory "+child.FullName+" to the listStore");
						TreeIter childIter = dirTree.AppendValues(iter, child);
						if (child.hasSubDirectories)
						{
							//Console.WriteLine("     "+"Directory "+child.FullName+" has subdirectories, making a dummy Dirinfo, for the little triangle");
							dirTree.AppendValues(childIter,new DirInfo());
						}
					}
				}
				// remove the dummy DirInfo that was appended when created the DirInfo, 
				// for the reason mentioned above
				// but it is only needed for 
				TreeIter tempIter;
				dirTree.IterChildren(out tempIter,iter);
				dirTree.Remove(ref tempIter);
			}
			//Console.WriteLine("Leaving FSReader.ReadDir");
		}
		
		/// <summary>
		/// Initiates the exploration of a directory,
		/// and appending its subdurectories to the DirectoryTree TreeStore
		/// </summary>
		/// <param name="path">
		/// A TreePath path set to the directory to be explored<see cref="TreePath"/>
		/// </param>
		public void ExploreDir(TreePath path)
		{
			//Console.WriteLine("Entering FSReader.ExploreDir");
			TreeIter iter = new TreeIter();
			dirTree.GetIter(out iter, path);
			ReadDir(((DirInfo)dirTree.GetValue(iter,0)),iter);
			//Console.WriteLine("Exiting FSReader.ExploreDir");
		}
		
		public void UnexploreDir(TreePath path)
		{
			//Console.WriteLine("Entering Sagittarius.FilesystemReader.UnexploreDir");
			TreeIter iter = new TreeIter();
			dirTree.GetIter(out iter, path);
			TreeIter childIter = new TreeIter();
			dirTree.IterChildren(out childIter,iter);
			while(dirTree.IterHasChild(iter))
			{
				dirTree.Remove(ref childIter);
			}
			dirTree.AppendValues(iter,new DirInfo()); //make a dummy DirInfo under it for the TreeView to display expanders near its icon
			//Console.WriteLine("Leaving Sagittarius.FilesystemReader.UnexploreDir");
		}
		
		public string GetDirName(TreePath path)
		{
			//Console.WriteLine("Entering FSReader.GetDirName");
			TreeIter iter = new TreeIter();
			dirTree.GetIter(out iter, path);
			//Console.WriteLine("Exiting FSReader.GetDirName");
			return ((DirInfo)dirTree.GetValue(iter,0)).FullName;
		}
		
		public void DeletePath(TreePath path)
		{
			//Console.WriteLine("Entering FileSystemReader.DeletePath");
			TreeIter iter = new TreeIter();
			dirTree.GetIter(out iter, path);
			dirTree.Remove(ref iter);
		}
		
		public void AddPath(string directory)
		{
			DirInfo root = new DirInfo(directory);
			TreeIter rootIter = dirTree.AppendValues(root); 
			ReadDir(root, rootIter);
		}
		
		public bool ShowHidden
		{
			get{ return showHidden; }
		}
		
		public TreeStore DirectoryTree
		{
			get { return dirTree; }
		}
	}	
}