/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using Gdk;
using System.Collections.Generic;
using System.Xml;
using System.IO;
using Sagittarius.Exiftool;
using Sagittarius.Config;

namespace Sagittarius
{
	public class CacheManager
	{
		private string directory;
		private string cacheDirectory;
		private string cacheFilename;
		private List<CacheInfo> cacheList;
		private bool thereAreUnsavedChanges=false;
		
		public CacheManager(string directory)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager costructor");
			this.directory=directory;
			cacheDirectory = Misc.CACHE_DIR+directory+"/";
			cacheFilename = Misc.CACHE_DIR+directory+"/cache.xml";
			ReadCacheOfDirectory();
			if(cacheList==null)
			{
				cacheList = new List<CacheInfo>();
			}
			else
			{
				CheckCache();
			}
			//Console.WriteLine("Leaving Sagittarius.CacheManager costructor");
		}
		
		private void ReadCacheOfDirectory()
		{ 
			//Console.WriteLine("Entering Sagittarius.CacheManager.ReadCacheOfDirectory");
			DirectoryInfo dirInfo = new DirectoryInfo(cacheDirectory);
			if(!dirInfo.Exists)
			{
				Directory.CreateDirectory(cacheDirectory);
				return;
			}
			dirInfo = null;

			cacheList = new List<CacheInfo>();
			
			FileInfo cacheFile = new FileInfo(cacheFilename);
			bool cacheFileExist = cacheFile.Exists;
			cacheFile=null;
			if(cacheFileExist)
			{ // a cache file létezik, lehet belőle olvasni
				//Console.WriteLine(" CacheFile exist, reading");
				XmlDocument cacheXml = new XmlDocument(); // the documnet that will be readed
	 			cacheXml.Load(cacheFilename); // load the file
	 			XmlTextReader xr = new XmlTextReader(cacheFilename); // create a XmlTextREader in order to read file's content
				while(xr.Read())
				{  // start the main loop through the cachefile nodes of the file 
					if(xr.NodeType == XmlNodeType.Element) 
					{
						if(xr.Name=="cache")
						{ // if the element name is thumbnail then
							CacheInfo cache = new CacheInfo();
							while(xr.MoveToNextAttribute())
							{ // read all the attributes and..
								if(xr.Name=="originalFilename") 
								{ 
									cache.originalFilename=xr.Value;
									continue;
								} 
								if(xr.Name=="modifyTime")
								{
									cache.modifyTime=xr.Value;
									continue;
								}
								if(xr.Name=="thumbFilename")
								{ 
									cache.thumbFilename=xr.Value; 
									continue;
								}
								if(xr.Name=="prevFilename")
								{ 
									cache.previewFilename=xr.Value; 
									continue;
								}
								if(xr.Name=="thumbSize")
								{ 
									cache.thumbSize=xr.Value; 
									continue;
								}
								if(xr.Name=="prevSize")
								{
									cache.previewSize=xr.Value; 
									continue;
								}
							}
							cacheList.Add(cache);
						}
					}
				}
				xr.Close(); // close the XmlTextReader
			}
			//Console.WriteLine("Leaving Sagittarius.CacheManager.ReadCacheOfDirectory");
		}
		
		private void CheckCache()
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.CheckCache");
			string[] files = System.IO.Directory.GetFiles(directory); // all the files in current folder
			for(int k=0;k<files.Length;k++)
			{
				if(Misc.IsFileSupported(files[k])) // if current file is a supported image
				{
					CheckCacheOfFile(files[k]);
				}
			}
			//Console.WriteLine("Leaving Sagittarius.CacheManager.CheckCache");
		}
		
		private void CheckCacheOfFile(string filename)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.CheckCacheOfFile");
			CacheInfo actualCache= new CacheInfo();
			bool cacheIsUpToDate=false;
			bool cacheFound = false;
			//Console.WriteLine(" Searching for the record in the cache for the file: "+filename);
			foreach(CacheInfo cache in cacheList)
			{
				if(cache.originalFilename==filename)
				{
					//Console.WriteLine("  found");
					cacheFound = true;
					actualCache=cache;
					FileInfo fileInfo = new FileInfo(filename);
					if(Convert.ToDateTime(cache.modifyTime)>=fileInfo.LastWriteTime)
					{
						//Console.WriteLine("   modifyTime of the file is not newer than that of the cache, good");
						if(Convert.ToInt32(cache.previewSize)==Config.Preferences.PrevMaxSize)
						{
							//Console.WriteLine("   previewSize is appropriate, good");
							if(Convert.ToInt32(cache.thumbSize)==Config.Preferences.ThumbSize)
							{
								//Console.WriteLine("   thumbSize is appropriate, good");
								cacheIsUpToDate=true;
							}
						}
					}
					break;
				}
			}
			if(cacheFound&&(!cacheIsUpToDate))
			{
				//Console.WriteLine(" Cache is found, but is bad, removing");
				cacheList.Remove(actualCache);
				try
				{
					File.Delete(actualCache.thumbFilename);
					if(actualCache.previewFilename!="")
					{
						File.Delete(actualCache.previewFilename);
					}
				}
				catch(ArgumentNullException)
				{}
			}
			//Console.WriteLine("Leaving Sagittarius.CacheManager.CheckCacheOfFile");
		}
		
		public void WriteCacheForDirectory()
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.WriteCacheForDirectory");
			XmlDocument cacheXml = new XmlDocument(); 
			XmlElement xe = cacheXml.CreateElement("sagittarius-cache"); // The root element of the xml settings file
			cacheXml.AppendChild(xe); // append the root element
			foreach(CacheInfo cache in cacheList)
			{
				XmlElement cacheElement = cacheXml.CreateElement("cache");
				cacheElement.SetAttribute("originalFilename",cache.originalFilename);
				cacheElement.SetAttribute("modifyTime",cache.modifyTime);
				cacheElement.SetAttribute("thumbFilename",cache.thumbFilename);
				cacheElement.SetAttribute("prevFilename",cache.previewFilename);
				cacheElement.SetAttribute("thumbSize",cache.thumbSize);
				cacheElement.SetAttribute("prevSize",cache.previewSize);
				xe.AppendChild(cacheElement);
			}
			cacheXml.Save(cacheFilename);
			thereAreUnsavedChanges=false;
			//Console.WriteLine("Leaving Sagittarius.CacheManager.WriteCacheForDirectory");
		}
		
		public CacheInfo GetCacheForFile(string filename)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.GetCacheForFile");
			foreach(CacheInfo cache in cacheList)
			{
				if (cache.originalFilename==filename)
				{
					//Console.WriteLine("Leaving Sagittarius.CacheManager.GetCacheForFile");
					return cache;
				}
			}
			//Console.WriteLine("Leaving Sagittarius.CacheManager.GetCacheForFile");
			return null;
		}

		public void AddCache(string filename, Pixbuf thumbnail)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.AddCache");
			int pos = filename.LastIndexOf("/");
			string name = filename.Substring(pos+1,filename.Length-pos-1);
			CacheInfo cInfo = new CacheInfo();
			cInfo.modifyTime=new FileInfo(filename).LastWriteTime.ToString();
			cInfo.originalFilename= filename;
			cInfo.previewFilename="";
			cInfo.previewSize=Config.Preferences.PrevMaxSize.ToString();
			cInfo.thumbFilename=Misc.CACHE_DIR+directory+"/"+name+"_thumb.png";
			cInfo.thumbSize=Config.Preferences.ThumbSize.ToString();
			thumbnail.Save(cInfo.thumbFilename,"png");
			cacheList.Add(cInfo);
			thereAreUnsavedChanges=true;
			//Console.WriteLine("Leaving Sagittarius.CacheManager.AddCache");
		}
		
		public void AddCache(string filename, Pixbuf preview, Pixbuf thumbnail)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.AddCache");
			int pos = filename.LastIndexOf("/");
			string name = filename.Substring(pos+1,filename.Length-pos-1);
			CacheInfo cInfo = new CacheInfo();
			cInfo.modifyTime=new FileInfo(filename).LastWriteTime.ToString();
			cInfo.originalFilename= filename;
			cInfo.previewFilename=Misc.CACHE_DIR+directory+"/"+name+"_prev.png";
			cInfo.previewSize=Config.Preferences.PrevMaxSize.ToString();
			cInfo.thumbFilename=Misc.CACHE_DIR+directory+"/"+name+"_thumb.png";
			cInfo.thumbSize=Config.Preferences.ThumbSize.ToString();
			preview.Save(cInfo.previewFilename,"png");
			thumbnail.Save(cInfo.thumbFilename,"png");
			cacheList.Add(cInfo);
			thereAreUnsavedChanges=true;
			//Console.WriteLine("Leaving Sagittarius.CacheManager.AddCache");
		}
		
		public void AddPreviewCache(string filename, Pixbuf preview)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.AddPreviewCache");
			CacheInfo cInfo = GetCacheForFile(filename);
			int pos = filename.LastIndexOf("/");
			string name = filename.Substring(pos+1,filename.Length-pos-1);
			string previewFilename= Misc.CACHE_DIR+directory+"/"+name+"_prev.png";

			preview.Save(previewFilename,"png");
			cInfo.previewFilename=previewFilename;
			//Console.WriteLine("Leaving Sagittarius.CacheManager.AddPreviewCache");
		}
		
		public bool Unsaved
		{ 
			get { return thereAreUnsavedChanges; }
		}
		
		public void UpdateCacheDate(string filename, string newDate)
		{
			//Console.WriteLine("Entering Sagittarius.CacheManager.UpdateCacheDate");
			CacheInfo cInfo = this.GetCacheForFile(filename);
			cInfo.modifyTime=newDate;
			//Console.WriteLine("Leaving Sagittarius.CacheManager.UpdateCacheDate");
		}
	}
	
}
