///* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
// * 
// * This file is part of Sagittarius.
// *
// * Sagittarius is free software: you can redistribute it and/or modify
// * it under the terms of the GNU General Public License as published by
// * the Free Software Foundation, either version 2 of the License, or
// * (at your option) any later version.
// *
// * Sagittarius is distributed in the hope that it will be useful,
// * but WITHOUT ANY WARRANTY; without even the implied warranty of
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// * GNU General Public License for more details.
// *
// * You should have received a copy of the GNU General Public License
// * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
// */

using System;
using Gdk;
using Gtk;
using Sagittarius.Metadata;
using Sagittarius.Exiftool;
using System.IO;
using System.Threading;

namespace Sagittarius
{
	public class PhotoInfo
	{
		private string filename;
		private Gdk.Pixbuf thumbnail;
		private Gdk.Pixbuf preview;
		private XmpMetadata metadata;
		private CacheManager cManager;
		private string markup;
		private Gtk.ThreadNotify threadNotify;
		private Thread previewLoaderThread;
		private string fileSize;

		public PhotoInfo(string filename, CacheManager cManager)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo constructor for file: "+filename);
			this.filename=filename;
			this.cManager=cManager;
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo constructor for file: "+filename);
		}
		
		public void ReadMetadata(XmpMetadata metadata)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.ReadMetadata for file: "+filename);
			this.metadata= metadata;
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.ReadMetadata for file: "+filename);
		}
		
		public void WriteMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.WriteMetadata");
			MetadataParser.WriteMetadata(metadata,filename);
			metadata.SaveChanges();
			cManager.UpdateCacheDate(filename, new FileInfo(filename).LastWriteTime.ToString());
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.WriteMetadata");
		}
		
		public void DiscardMetadataChanges ()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.DiscardMetadataChanges");
			metadata.DiscardChanges();
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.DiscardMetadataChanges");
		}

		public /*Pixbuf*/void BeginLoadPreview()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.GetPreview");
			//Console.WriteLine(" checking if the preview is already extracted or not... ");
			if(preview==null)
			{
				//Console.Write(" NOT EXTRACTED, extracting");
				//preview = new Pixbuf(null,"null.png");
				preview = null;
				ThreadStart tStart = new ThreadStart (LoadPreview);
				this.previewLoaderThread = new Thread (tStart);
				this.previewLoaderThread.Start ();
				threadNotify = new Gtk.ThreadNotify (new Gtk.ReadyEvent (LoadPreviewFinished));
			}
			else
			{
				LoadPreviewFinished();
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.GetPreview");
			//return preview;
		}
		
		private void LoadPreview()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.LoadPreview");
			using (Pixbuf pixbuf = new Pixbuf (filename)) 
			{
				GeneratePreview(pixbuf);
			}
			cManager.AddPreviewCache(filename,preview);
			threadNotify.WakeupMain ();
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.LoadPreview");
		}
		
		private void LoadPreviewFinished()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.LoadPreviewFinished");
			if (OnPreviewLoaded != null)
			{
				OnPreviewLoaded (this, new PreviewLoadedEventArgs (Preview,this.Filename));
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.LoadPreviewFinished");
		}

		private void GeneratePreview(Pixbuf pixbuf)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.GeneratePreview");
			int photoWidth = pixbuf.Width;
			int photoHeight = pixbuf.Height;
			double prop=(double)photoHeight/(double)photoWidth;
			
			if((photoWidth>Config.Preferences.PrevMaxSize)||(photoHeight>Config.Preferences.PrevMaxSize))
			{
				if(photoWidth>=photoHeight)
				{
					preview = pixbuf.ScaleSimple(Config.Preferences.PrevMaxSize,(int)(prop*Config.Preferences.PrevMaxSize),InterpType.Bilinear);
				}
				else
				{
					preview = pixbuf.ScaleSimple((int)(Config.Preferences.PrevMaxSize/prop),Config.Preferences.PrevMaxSize,InterpType.Bilinear);
				}
			}
			else
			{
				preview = pixbuf.Copy();
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.GeneratePreview");
		}
		
		private void CreateMarkup()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.CreateMarkup for file: "+filename);
			markup=
				"Filename: <b>"+this.ShortFilename+"</b>\n"+
				"File size: <b>"+FileSize+"</b>\n"+
				"Image caption: <b>"+metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DESCRIPTION)+"</b>";
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.CreateMarkup for file: "+filename);
		}
		
		
		public string Markup
		{
			get 
			{
				//Console.WriteLine("Entering Sagittarius.PhotoInfo.Markup for file: "+filename);
				if (markup== null)
				{
					CreateMarkup();
				}
				//Console.WriteLine("Leaving Sagittarius.PhotoInfo.Markup for file: "+filename);
				return markup; 
			}
		}

		public string FileSize
		{
			get
			{
				if(fileSize==null)
				{
					long size= new FileInfo(Filename).Length;
					if(size<1000) 
					{
						fileSize= size.ToString() + " byte";
					}
					else if(size>=1000) 
					{
						fileSize= Math.Abs(size/1000).ToString() + " Kb";
					}
				}
				return fileSize;
			}
		}

		public string Filename
		{
			get { return filename; }
			set { filename = value; }
		}
		
		public Gdk.Pixbuf Thumbnail
		{
			get { return thumbnail; }
			set { thumbnail = value; }
		}
		
		public Gdk.Pixbuf Preview
		{
			get { return preview; }
			set { preview = value; }
		}
		
		public XmpMetadata Metadata
		{
			get { return metadata; }
		}
		
		public bool MetadataChanged
		{
			get 
			{ return metadata.Changed; }
		}
		
		public bool NewXmp
		{
			get { return metadata.NewXmp; }
			set { metadata.NewXmp = value; }
		}
		
		public string ShortFilename
		{
			get
			{
				int pos = Filename.LastIndexOf("/");
				return Filename.Substring(pos+1,filename.Length-pos-1);
			}
		}

		public delegate void OnPreviewLoadedHandler (PhotoInfo o, PreviewLoadedEventArgs args);
		public event OnPreviewLoadedHandler OnPreviewLoaded;

	}
}
