/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using Sagittarius.Exiftool;
using Sagittarius.Metadata;
using System.Collections;
using System.Threading;
using Gdk;
using System.IO;

namespace Sagittarius
{
	public class PhotoInfoFactory
	{

		public static ArrayList threadsToStop;

		private Queue photoQueue;
		private Queue photoDone;
		private Thread factoryWorker;
		private FactorySemaphore factorySemaphore;
		private Gtk.ThreadNotify threadNotify;
		private CacheManager cacheManager;

		static PhotoInfoFactory ()
		{
			threadsToStop = new ArrayList ();
		}

		public static void KillThreads ()
		{
			foreach (Thread t in threadsToStop) 
			{
				t.Abort ();
			}
		}

		public PhotoInfoFactory(CacheManager cacheManager)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfoFactory constructor");
			this.cacheManager=cacheManager;
			photoQueue = new Queue ();
			photoDone = new Queue ();
			factorySemaphore = new FactorySemaphore ();

			ThreadStart tStart = new ThreadStart (FactoryThread);
			factoryWorker = new Thread (tStart);
			factoryWorker.Start ();

			threadsToStop.Add (factoryWorker);

			threadNotify = new Gtk.ThreadNotify (new Gtk.ReadyEvent (PhotoInfoFinished));
			//Console.WriteLine("Leaving Sagittarius.PhotoInfoFactory constructor");
		}

		public void BeginProcessPhoto (string filename)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfoFactory.BeginProcessPhoto for file: "+filename);
			// a stack nem thread_safe, ezért lockolni kell
			lock (photoQueue) 
			{
				// ha a stackben nincs benne az adott photoInfo ojjekt, 
				//if (!photoStack.Contains (pInfo)) 
				//{
				PhotoInfo pInfo = new PhotoInfo(filename,cacheManager);
					// akkor belerakjuk
					photoQueue.Enqueue (pInfo);
					// majd értesítjük a sorban álló thread-eket, 
					// hogy a szemafor szabad ???
					factorySemaphore.Up ();
				//}
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfoFactory.BeginProcessPhoto for file: "+filename);
		}

		private void PhotoInfoFinished ()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfoFactory.ProcessFinished");
			lock (photoDone) 
			{
				while (photoDone.Count > 0) 
				{
					PhotoInfo pInfo = (PhotoInfo) photoDone.Dequeue ();
					if (OnPhotoInfoReady != null)
					{
						OnPhotoInfoReady (this, new PhotoInfoReadyEventArgs (pInfo));
					}
				}
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfoFactory.ProcessFinished");
		}
		
		private void FactoryThread ()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfoFactory.FactoryThread");
			while (true) 
			{
				try 
				{
					factorySemaphore.Down ();
				} 
				catch 
				{}

				PhotoInfo pInfo;

				while (photoQueue.Count > 0) 
				{
					lock (photoQueue) 
					{
						pInfo = (PhotoInfo) photoQueue.Dequeue ();
					}

					ProcessPhoto (pInfo);

					lock (photoDone) 
					{
						photoDone.Enqueue (pInfo);
					}
					threadNotify.WakeupMain ();
				}
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfoFactory.FactoryThread");
		}
		
		private void ProcessPhoto (PhotoInfo pInfo)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfoFactory.ProcessPhoto for file: "+pInfo.Filename);
			CacheInfo cInfo = cacheManager.GetCacheForFile(pInfo.Filename);
			//Console.WriteLine("Checking if "+pInfo.Filename+" is already cached or not...");
			bool newXmp;
			pInfo.ReadMetadata(MetadataParser.GetMetadata(pInfo.Filename,out newXmp));
			pInfo.NewXmp=newXmp;
			//pInfo.Metadata = new XmpMetadata(pInfo.Filename);
			if(cInfo==null)
			{
				//Console.WriteLine(".it is not cached :-(");
				using (Pixbuf pixbuf = new Pixbuf (pInfo.Filename)) 
				{
					GenerateThumb(pInfo,pixbuf);
				}
				cacheManager.AddCache(pInfo.Filename,pInfo.Thumbnail);
			}
			else
			{
				//Console.WriteLine(".it is cached :-)");
				//Console.WriteLine(".prevfilename:"+cInfo.previewFilename);
				if(cInfo.previewFilename!="")
				{
					//Console.WriteLine(".. even the preview is cached");
					pInfo.Preview=new Pixbuf(cInfo.previewFilename);
				}
				pInfo.Thumbnail= new Pixbuf(cInfo.thumbFilename);
			}

			//Console.WriteLine("Leaving Sagittarius.PhotoInfoFactory.ProcessPhoto for file: "+pInfo.Filename);
		}

		private void GenerateThumb(PhotoInfo pInfo, Pixbuf pixbuf)
		{
			//Console.WriteLine("Entering Sagittarius.ThumbDataFactory.GenerateThumb");
			int photoWidth= pixbuf.Width;
			int photoHeight= pixbuf.Height;
			int thumbWidth;
			int thumbHeight;
			
			if(photoWidth>=photoHeight)
			{
				thumbHeight=(int)((float)photoHeight/(float)photoWidth*Config.Preferences.ThumbSize);
				thumbWidth=Config.Preferences.ThumbSize;
			}
			else
			{
				thumbWidth=(int)((float)photoWidth/(float)photoHeight*Config.Preferences.ThumbSize);
				thumbHeight=Config.Preferences.ThumbSize;
			}
			pInfo.Thumbnail = pixbuf.ScaleSimple(thumbWidth,thumbHeight,InterpType.Bilinear);
			
			//Console.WriteLine("Leaving Sagittarius.ThumbDataFactory.GenerateThumb");
		}
		
		
		public delegate void OnPhotoInfoReadyHandler (PhotoInfoFactory o, PhotoInfoReadyEventArgs args);
		public event OnPhotoInfoReadyHandler OnPhotoInfoReady;
		
	}
}
				