/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using Gtk;
using Gdk;
using Sagittarius.Metadata;
using Sagittarius.Exiftool;
using Sagittarius.Config;

namespace Sagittarius
{
	public partial class MainWindow: Gtk.Window
	{
		private FileSystemReader fsReader, bookmarkReader; // an object responsible for the data of the DirectoryTree;
		private BookmarkManager bmManager;
		private int prevBoxWidth, prevBoxHeight; // size of the imageWidget (before resizing)
		private XmpMetadata metadata;
		private ListStore photoStore;
		private List<PhotoInfo> selectedPhotos;
		private PhotoInfo activePhoto;
		private CacheManager cacheManager;
		private int numOfPhotosInDirectory, numOfPhotosLoaded;

#region setup
		
		public MainWindow (): base (Gtk.WindowType.Toplevel)
		{
			Build ();
            this.imaDatPic.Changed += new Sagittarius.Widgets.DateEdit.ChangedHandler(this.OnImaDatPicChanged);
			
			bmManager = new BookmarkManager();

			// theese variables are used during preview resizing
			prevBoxWidth = previewWidget.Allocation.Width;
			prevBoxHeight = previewWidget.Allocation.Height;

			//SetupFileSystemTreeView()
			fsReader = new FileSystemReader(false);
			TreeViewColumn iconCol = new TreeViewColumn();
			iconCol.Sizing=TreeViewColumnSizing.Autosize;
			CellRendererPixbuf iconCell = new CellRendererPixbuf();
			iconCol.PackStart(iconCell,false);
			CellRendererText nameCell = new CellRendererText();
			iconCol.PackStart(nameCell,true);
			fsTreeView.AppendColumn(iconCol);
			iconCol.SetCellDataFunc(iconCell,new Gtk.TreeCellDataFunc(RenderFsTreeViewDirIcon));
			iconCol.SetCellDataFunc(nameCell,new Gtk.TreeCellDataFunc(RenderFsTreeViewDirName));
			fsTreeView.Model=fsReader.DirectoryTree;
			fsTreeView.HeadersVisible=false;
			
			//SetupBookmarkTreeView()
			iconCol = new TreeViewColumn();
			iconCol.Sizing=TreeViewColumnSizing.Autosize;
			iconCell = new CellRendererPixbuf();
			iconCol.PackStart(iconCell,false);
			nameCell = new CellRendererText();
			iconCol.PackStart(nameCell,true);
			bookmarkTreeView.AppendColumn(iconCol);
			iconCol.SetCellDataFunc(iconCell,new Gtk.TreeCellDataFunc(RenderBookmarkTreeViewDirIcon));
			iconCol.SetCellDataFunc(nameCell,new Gtk.TreeCellDataFunc(RenderBookmarkTreeViewDirName));
			//bookmarkTreeView.HeadersVisible=false;
			bookmarkReader = new FileSystemReader(false,bmManager.BookmarkList);
			bookmarkTreeView.Model=bookmarkReader.DirectoryTree;
			
			//SetupThumbView()
			TreeViewColumn thumbCol = new TreeViewColumn();
			CellRendererPixbuf thumbCell = new CellRendererPixbuf();
			thumbCol.AddAttribute(thumbCell, "pixbuf", 0);  // it will display an image on the left (the image thumb) ....
			thumbCol.PackStart(thumbCell, false);
			thumbCol.SetCellDataFunc(thumbCell, new TreeCellDataFunc(RenderThumbnail));
			thumbView.AppendColumn (thumbCol);
			TreeViewColumn markupCol = new TreeViewColumn ();
			CellRenderer markupCell = new CellRendererText();
			markupCol.AddAttribute (markupCell, "markup", 1);  // ...and a lot of formatted information
			markupCol.PackStart(markupCell, true); 
			markupCol.SetCellDataFunc(markupCell, new TreeCellDataFunc(RenderMarkup));
			thumbView.AppendColumn (markupCol);
			thumbView.HeadersVisible=false;
			thumbView.Selection.Mode=SelectionMode.Multiple;
			thumbView.Selection.Changed +=  new EventHandler(OnThumbViewSelectionChanged);
			
			//setupMetadataNotebook
			TreeViewColumn keyCol = new TreeViewColumn();
			keyCol.Title="Key";
			CellRendererText keyCell = new CellRendererText();
			keyCol.PackStart(keyCell,true);
			TreeViewColumn valueCol = new TreeViewColumn();
			valueCol.Title = "Value";
			CellRendererText valueCell = new CellRendererText();
			valueCol.PackStart(valueCell,true);
			exifTreeView.AppendColumn(keyCol);
			exifTreeView.AppendColumn(valueCol);
			keyCol.AddAttribute(keyCell,"text",0);
			valueCol.AddAttribute(valueCell,"text",1);
		}
		
		private void ResetMainWindow()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.ResetMainWindow");
			metadata = null;
			previewWidget.Pixbuf=new Pixbuf(null,"null.png");
			thumbView.Model=null;

			creCreEnt.Text="";
			creJobEnt.Text="";
			creAddText.Buffer.Text="";
			creCitEnt.Text="";
			creStaEnt.Text="";
			crePosEnt.Text="";
			creCouEnt.Text="";
			creTelText.Buffer.Text="";
			creEmaText.Buffer.Text="";
			creWebText.Buffer.Text="";
			conHeaText.Buffer.Text="";
			conCapText.Buffer.Text="";
			conIptText.Buffer.Text="";
			conWriEnt.Text="";
			conKeyText.Buffer.Text="";
			imaDatPic.CurrentDateTime=new MDate(new DateTime(),0);
			imaIntEnt.Text="";
			imaIptText.Buffer.Text="";
			imaLocEnt.Text="";
			imaCitEnt.Text="";
			imaStaEnt.Text="";
			imaCouEnt.Text="";
			imaIsoEnt.Text="";
			staTitEnt.Text="";
			staJobEnt.Text="";
			staInsText.Buffer.Text="";
			staProEnt.Text="";
			staSouEnt.Text="";
			copStaCom.Active=2;
			copCopText.Buffer.Text="";
			copRigText.Buffer.Text="";
			
			exifTreeView.Model=null;
			//Console.WriteLine("Entering Sagittarius.MainWindow.ResetMainWindow");
		}

		private void Quit()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.Quit");
			AskForSave();
			if(cacheManager!=null)
			{
				cacheManager.WriteCacheForDirectory();
			}
			PhotoInfoFactory.KillThreads();
			//Console.WriteLine("Leaving Sagittarius.MainWindow.Quit");
			Application.Quit();
		}

#endregion
		
#region fstreeview, bookmarktreeview, load
		
		private void RenderFsTreeViewDirName(Gtk.TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			DirInfo dirInfo = (DirInfo) model.GetValue(iter,0);
			(cell as Gtk.CellRendererText).Text = dirInfo.Name;
		}
		
		private void RenderFsTreeViewDirIcon(Gtk.TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			DirInfo dirInfo = (DirInfo) model.GetValue(iter,0);
			(cell as Gtk.CellRendererPixbuf).Pixbuf = dirInfo.Icon;
		}

		private void RenderBookmarkTreeViewDirName(Gtk.TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			DirInfo dirInfo = (DirInfo) model.GetValue(iter,0);
			(cell as Gtk.CellRendererText).Text = dirInfo.Name;
		}
		
		private void RenderBookmarkTreeViewDirIcon(Gtk.TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			DirInfo dirInfo = (DirInfo) model.GetValue(iter,0);
			(cell as Gtk.CellRendererPixbuf).Pixbuf = dirInfo.Icon;
		}

		protected virtual void OnFsTreeViewRowActivated (object o, Gtk.RowActivatedArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.OnFsTreeViewRowActivated");
			fsTreeView.GrabFocus();
			AskForSave();
			ResetMainWindow();
			thumbView.Model=ReadDirectoryContents(fsReader.GetDirName(args.Path));
			thumbView.ColumnsAutosize();
			//Console.WriteLine("Exiting Sagittarius.MainWindow.OnFsTreeViewRowActivated");
		}

		protected virtual void OnBookmarkTreeViewRowActivated (object o, Gtk.RowActivatedArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow..OnBookmarkTreeViewRowActivated");
			bookmarkTreeView.GrabFocus();
			AskForSave();
			ResetMainWindow();
			thumbView.Model=ReadDirectoryContents(bookmarkReader.GetDirName(args.Path));
			//Console.WriteLine("Leaving Sagittarius.MainWindow..OnBookmarkTreeViewRowActivated");
		}
		
		protected virtual void OnFsTreeViewRowExpanded (object o, Gtk.RowExpandedArgs args)
		{
			fsTreeView.GrabFocus();
			fsReader.ExploreDir(args.Path);
		}

		protected virtual void OnFsTreeViewRowCollapsed (object o, Gtk.RowCollapsedArgs args)
		{
			fsTreeView.GrabFocus();
			fsReader.UnexploreDir(args.Path);
		}

		protected virtual void OnBookmarkTreeViewRowExpanded (object o, Gtk.RowExpandedArgs args)
		{
			bookmarkTreeView.GrabFocus();
			bookmarkReader.ExploreDir(args.Path);
		}

		protected virtual void OnBookmarkTreeViewRowCollapsed (object o, Gtk.RowCollapsedArgs args)
		{
			bookmarkTreeView.GrabFocus();
			bookmarkReader.UnexploreDir(args.Path);
		}

		protected virtual void OnAddBookmarkBtnClicked (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering MainWindow.OnAddBookmarkBtnClicked");
			addBookmarkBtn.GrabFocus();
			TreePath[] path =fsTreeView.Selection.GetSelectedRows();
			if(path.Length>0)
			{
				bmManager.AddBookmark(fsReader.GetDirName(path[0]));
				bmManager.SaveBookmarks(false);
				bookmarkReader.AddPath(fsReader.GetDirName(path[0]));
			}
			//Console.WriteLine("Leaving MainWindow.OnAddBookmarkBtnClicked");
		}

		protected virtual void OnRemoveBookmarkBtnClicked (object sender, System.EventArgs e)
		{
			removeBookmarkBtn.GrabFocus();
			TreePath[] path =bookmarkTreeView.Selection.GetSelectedRows();
			if (path.Length>0)
			{
				for (int i= path[0].Depth;i>1;i--)
				{
					path[0].Up();
				}
				bmManager.DeleteBookmark(bookmarkReader.GetDirName(path[0]));
				bmManager.SaveBookmarks(false);
				bookmarkReader.DeletePath(path[0]);
			}
		}

		public Gtk.ListStore ReadDirectoryContents(string path)
		{
			//Console.WriteLine("Entering Sagittarius.Main.ReadDirectoryContents");
			if(cacheManager!=null)
			{
				cacheManager.WriteCacheForDirectory();
			}
			photoStore = new ListStore(typeof(PhotoInfo));
			string[] files = System.IO.Directory.GetFiles(path); // all the files in current folder
			numOfPhotosInDirectory = 0;
			numOfPhotosLoaded=0;
			cacheManager = new CacheManager(path);
			PhotoInfoFactory.KillThreads();
			PhotoInfoFactory factory = new PhotoInfoFactory(cacheManager);
			factory.OnPhotoInfoReady+=new PhotoInfoFactory.OnPhotoInfoReadyHandler (PhotoInfoIsAvailable);
			for(int k=0;k<files.Length;k++)
			{
				if(Config.Misc.IsFileSupported(files[k])) // if current file is a supported image
				{
					numOfPhotosInDirectory++;
					factory.BeginProcessPhoto(files[k]);
				}
				//Console.WriteLine(numOfPhotosInDirectory);
			}
			//Console.WriteLine("Leaving Sagittarius.Main.ReadDirectoryContents");
			return photoStore;
		}

		private void PhotoInfoIsAvailable(PhotoInfoFactory factory, PhotoInfoReadyEventArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.PhotoInfoIsAvailable");
			photoStore.AppendValues(args.PInfo);
			numOfPhotosLoaded++;
			LoadSaveStatusUpdated(numOfPhotosLoaded,numOfPhotosInDirectory,"Loading photos...");
			//Console.WriteLine("Leaving Sagittarius.MainWindow.PhotoInfoIsAvailable");
		}
		
		private void LoadSaveStatusUpdated(int act, int max, string message)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.LoadSaveStatusUpdated");
			thumbStatusBox.Visible=true;
			progressbar.Adjustment.Lower=0;
			progressbar.Adjustment.Upper=max;
			progressbar.Adjustment.Value=act;
			progressbar.Text=Math.Round(((double)act/(double)max)*100).ToString()+"%";
			statusLabel.Text=message;
			if(max==act)
			{
				statusLabel.Text="";
				thumbStatusBox.Visible=false;
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.LoadSaveStatusUpdated");
		}

#endregion
		
#region thumbview
		
		private void RenderThumbnail (Gtk.TreeViewColumn columnt,Gtk.CellRenderer cell, Gtk.TreeModel model,Gtk.TreeIter iter)
		{
			PhotoInfo currentPInfo = (PhotoInfo) model.GetValue(iter,0);
			((CellRendererPixbuf) cell).Pixbuf = currentPInfo.Thumbnail;
		}

		private void RenderMarkup (Gtk.TreeViewColumn columnt,Gtk.CellRenderer cell, Gtk.TreeModel model,Gtk.TreeIter iter)
		{
			PhotoInfo currentPInfo = (PhotoInfo) model.GetValue(iter,0);
			((CellRendererText) cell).Markup = currentPInfo.Markup;
		}

		private void OnThumbViewSelectionChanged (object o, EventArgs args)
		{
			// az első lépésben megállapítjuk a kurzor helyét
			TreePath path;
			TreeViewColumn col;
			thumbView.GetCursor(out path, out col);
			if(path!=null)
			{
				// ebből megállapítjuk az aktív fotót
				TreeIter iter;
				photoStore.GetIter (out iter, path);
				activePhoto = (PhotoInfo) photoStore.GetValue(iter,0);
				metadata = activePhoto.Metadata;
				//Console.WriteLine(activePhoto.Filename);
			}
			TreeModel model;
 			TreePath[] selectedPaths = ((TreeSelection)o).GetSelectedRows(out model);
 			if (selectedPaths.Length>0)
 			{
				selectedPhotos=new List<PhotoInfo>();
				TreeIter[] iters = new TreeIter[selectedPaths.Length];
				for (int i=0; i<iters.Length;i++)
				{
					photoStore.GetIter (out iters[i],selectedPaths[i]);
					selectedPhotos.Add(((PhotoInfo) photoStore.GetValue(iters[i],0)));
				}
				GatherMetadata();
 			}
			if(metadata!=null)
			{
				ShowMetadata();
			}
			if(activePhoto!=null)
			{
				ShowPreview();
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.OnThumbViewSelectionChanged");
		}
		
#endregion
		
#region preview 
		
		private void ShowPreview()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.ShowPreview");
			if(prevBoxWidth>Config.Preferences.PrevMaxSize)
			{
				prevBoxWidth = Config.Preferences.PrevMaxSize;
			}
			if(prevBoxHeight > Config.Preferences.PrevMaxSize)
			{
				prevBoxHeight = Config.Preferences.PrevMaxSize;
			}
			activePhoto.OnPreviewLoaded+=new PhotoInfo.OnPreviewLoadedHandler(ActivePhotoPreviewLoaded);
			activePhoto.BeginLoadPreview();
			//PutPreview(activePhoto.BeginLoadPreview());
			//Console.WriteLine("Leaving Sagittarius.MainWindow.ShowPreview");
		}
		
		private void ActivePhotoPreviewLoaded(PhotoInfo o, Sagittarius.PreviewLoadedEventArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.ActivePhotoPreviewLoaded");
			//if(args.Filename==activePhoto.Filename)
			//{
			PutPreview(args.Preview);
			activePhoto.OnPreviewLoaded-=ActivePhotoPreviewLoaded;
			//}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.ActivePhotoPreviewLoaded");
		}

		protected virtual void OnPreviewWidgetExposeEvent (object o, Gtk.ExposeEventArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.OnImageWidgetExpose");
			if (previewWidget.Pixbuf!=null)
			{
				//Console.WriteLine(" pixbuf in not null");
				if ((prevBoxWidth != previewWidget.Allocation.Width) || (prevBoxHeight != previewWidget.Allocation.Height))
				{
					//Console.WriteLine(" previewwidget size changed");
					prevBoxWidth = previewWidget.Allocation.Width;
					prevBoxHeight = previewWidget.Allocation.Height;

					PutPreview(activePhoto.Preview);
				}
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.OnImageWidgetExpose");
		}
		
		private void PutPreview(Pixbuf preview)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.PutPreview");
			int previewWidth,previewHeight;
			double boxProp = Convert.ToDouble(prevBoxWidth)/Convert.ToDouble(prevBoxHeight);
			double prop = (double)preview.Width/(double)preview.Height;
			
			if (prop>=boxProp)
			{
				// a kép szélesebb, szélességhez kell igazítani
				previewWidth = prevBoxWidth;
				previewHeight = Convert.ToInt32(preview.Height*Convert.ToDouble(prevBoxWidth)/Convert.ToDouble(preview.Width));
			}
			else
			{
				previewHeight = prevBoxHeight;
				previewWidth = Convert.ToInt32(preview.Width*Convert.ToDouble(prevBoxHeight)/Convert.ToDouble(preview.Height));
			}
			previewWidget.Pixbuf=preview.ScaleSimple(previewWidth,previewHeight,InterpType.Bilinear);
			//Console.WriteLine("Leaving Sagittarius.MainWindow.PutPreview");
		}
		
#endregion
		
#region metadata panel, metadata 
		
		public void ShowMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.ShowMetadata");
			if (metadata!=null)
			{
				creCreEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CREATOR);
				creJobEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CREATOR_JOB_TITLE);
				creAddText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_EXTADR);
				creCitEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_CITY);
				creStaEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_REGION);
				crePosEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_PCODE);
				creCouEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_CTRY);
				creTelText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_TEL_WORK);
				creEmaText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_EMAIL_WORK);
				creWebText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_URL_WORK);
				conHeaText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_HEADLINE);
				conCapText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DESCRIPTION);
				conIptText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_SUBJECT_CODE);
				conWriEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DESCRIPTION_WRITER);
				conKeyText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_KEYWORDS);
				imaDatPic.CurrentDateTime=(MDate)metadata.IptcSect.GetTagValue(TagName.IPTC_DATE_CREATED);
				imaIntEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_INTELLECTUAL_GENRE);
				imaIptText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_SCENE);
				imaLocEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_LOCATION);
				imaCitEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CITY);
				imaStaEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_PROVINCE_STATE);
				imaCouEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COUNTRY);
				imaIsoEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COUNTRY_CODE);
				staTitEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_TITLE);
				staJobEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_JOB_ID);
				staInsText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_INSTRUCTIONS);
				staProEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_PROVIDER);
				staSouEnt.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_SOURCE);
				copStaCom.Active=(int)metadata.IptcSect.GetTagValue(TagName.IPTC_COPYRIGHTED);
				copCopText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COPYRIGHT_NOTICE);
				copRigText.Buffer.Text=metadata.IptcSect.GetTagValueAsString(TagName.IPTC_RIGHT_USAGE_TERMS);

				MetadataSection exifSect = metadata.ExifSect;
				
				TreeStore exifTS = new TreeStore(typeof (string), typeof (string));

				Gtk.TreeIter iter = exifTS.AppendValues("Gear");
				Gtk.TreeIter iter2 = exifTS.AppendValues(iter,"Camera","");
				exifTS.AppendValues(iter2,TagName.EXIF_MAKE,exifSect.GetTagValue(TagName.EXIF_MAKE));
				exifTS.AppendValues(iter2,TagName.EXIF_MODEL,exifSect.GetTagValue(TagName.EXIF_MODEL));
				exifTS.AppendValues(iter2,TagName.EXIF_SERIAL,exifSect.GetTagValue(TagName.EXIF_SERIAL));
				exifTS.AppendValues(iter2,TagName.EXIF_FIRMWARE,exifSect.GetTagValue(TagName.EXIF_FIRMWARE));
				exifTS.AppendValues(iter2,TagName.EXIF_SOFTWARE,exifSect.GetTagValue(TagName.EXIF_SOFTWARE)); 
				iter2 = exifTS.AppendValues(iter,"Lens","");
				exifTS.AppendValues(iter2,TagName.EXIF_LENS,exifSect.GetTagValue(TagName.EXIF_LENS));
				exifTS.AppendValues(iter2,TagName.EXIF_MAX_APERTURE,exifSect.GetTagValue(TagName.EXIF_MAX_APERTURE));
				iter = exifTS.AppendValues("Exposure");
				iter2 = exifTS.AppendValues(iter,"Date","");
				exifTS.AppendValues(iter2,TagName.EXIF_DATETIME_ORIGINAL,     exifSect.GetTagValue(TagName.EXIF_DATETIME_ORIGINAL));
				exifTS.AppendValues(iter2,TagName.EXIF_DATETIME_DIGITIZED,    exifSect.GetTagValue(TagName.EXIF_DATETIME_DIGITIZED));
				iter2 = exifTS.AppendValues(iter,"Settings","");
				exifTS.AppendValues(iter2,TagName.EXIF_EXPOSURE_TIME,         exifSect.GetTagValue(TagName.EXIF_EXPOSURE_TIME));
				exifTS.AppendValues(iter2,TagName.EXIF_SHUTTER_SPEED         ,exifSect.GetTagValue(TagName.EXIF_SHUTTER_SPEED));
				exifTS.AppendValues(iter2,TagName.EXIF_APERTURE              ,exifSect.GetTagValue(TagName.EXIF_APERTURE));
				exifTS.AppendValues(iter2,TagName.EXIF_ISO                   ,exifSect.GetTagValue(TagName.EXIF_ISO));
				exifTS.AppendValues(iter2,TagName.EXIF_EXPOSURE_COMP         ,exifSect.GetTagValue(TagName.EXIF_EXPOSURE_COMP));
				exifTS.AppendValues(iter2,TagName.EXIF_EXPOSURE_PROG         ,exifSect.GetTagValue(TagName.EXIF_EXPOSURE_PROG));
				exifTS.AppendValues(iter2,TagName.EXIF_EXPOSURE_MODE         ,exifSect.GetTagValue(TagName.EXIF_EXPOSURE_MODE));
				exifTS.AppendValues(iter2,TagName.EXIF_METERING_MODE         ,exifSect.GetTagValue(TagName.EXIF_METERING_MODE));
				exifTS.AppendValues(iter2,TagName.EXIF_SCENE_CAPTURE         ,exifSect.GetTagValue(TagName.EXIF_SCENE_CAPTURE));
				iter2 = exifTS.AppendValues(iter,"Lens settings","");
				exifTS.AppendValues(iter2,TagName.EXIF_FOCAL_LENGTH          ,exifSect.GetTagValue(TagName.EXIF_FOCAL_LENGTH));
				exifTS.AppendValues(iter2,TagName.EXIF_FOCAL_LENGTH_35       ,exifSect.GetTagValue(TagName.EXIF_FOCAL_LENGTH_35));
				exifTS.AppendValues(iter2,TagName.EXIF_DIGITAL_ZOOM          ,exifSect.GetTagValue(TagName.EXIF_DIGITAL_ZOOM));
				exifTS.AppendValues(iter2,TagName.EXIF_FNUMBER               ,exifSect.GetTagValue(TagName.EXIF_FNUMBER ));
				iter2 = exifTS.AppendValues(iter,"Light and flash","");
				exifTS.AppendValues(iter2,TagName.EXIF_LIGHT_SOURCE          ,exifSect.GetTagValue(TagName.EXIF_LIGHT_SOURCE));
				exifTS.AppendValues(iter2,TagName.EXIF_WHITE_BALANCE         ,exifSect.GetTagValue(TagName.EXIF_WHITE_BALANCE));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_FIRED           ,exifSect.GetTagValue(TagName.EXIF_FLASH_FIRED));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_MODE            ,exifSect.GetTagValue(TagName.EXIF_FLASH_MODE));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_REDEYE          ,exifSect.GetTagValue(TagName.EXIF_FLASH_REDEYE));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_RETURN          ,exifSect.GetTagValue(TagName.EXIF_FLASH_RETURN));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_FUNCTION        ,exifSect.GetTagValue(TagName.EXIF_FLASH_FUNCTION));
				exifTS.AppendValues(iter2,TagName.EXIF_FLASH_ENERGY          ,exifSect.GetTagValue(TagName.EXIF_FLASH_ENERGY));
				iter = exifTS.AppendValues("Image");
				iter2 = exifTS.AppendValues(iter,"General","");
				exifTS.AppendValues(iter2,TagName.EXIF_IMAGE_WIDTH           ,exifSect.GetTagValue(TagName.EXIF_IMAGE_WIDTH));
				exifTS.AppendValues(iter2,TagName.EXIF_IMAGE_LENGTH          ,exifSect.GetTagValue(TagName.EXIF_IMAGE_LENGTH));
				exifTS.AppendValues(iter2,TagName.EXIF_ORIENTATION           ,exifSect.GetTagValue(TagName.EXIF_ORIENTATION));
				exifTS.AppendValues(iter2,TagName.EXIF_BITS_PER_SAMPLE       ,exifSect.GetTagValue(TagName.EXIF_BITS_PER_SAMPLE));
				exifTS.AppendValues(iter2,TagName.EXIF_COMPRESSION           ,exifSect.GetTagValue(TagName.EXIF_COMPRESSION));
				exifTS.AppendValues(iter2,TagName.EXIF_COLOR_SPACE           ,exifSect.GetTagValue(TagName.EXIF_COLOR_SPACE));
				exifTS.AppendValues(iter2,TagName.EXIF_RESOLUTION_X          ,exifSect.GetTagValue(TagName.EXIF_RESOLUTION_X));
				exifTS.AppendValues(iter2,TagName.EXIF_RESOLUTION_Y          ,exifSect.GetTagValue(TagName.EXIF_RESOLUTION_Y));
				exifTS.AppendValues(iter2,TagName.EXIF_RESOLUTION_UNIT       ,exifSect.GetTagValue(TagName.EXIF_RESOLUTION_UNIT));
				exifTS.AppendValues(iter2,TagName.EXIF_BRIGHTNESS            ,exifSect.GetTagValue(TagName.EXIF_BRIGHTNESS));
				iter2 = exifTS.AppendValues(iter,"Settings","");
				exifTS.AppendValues(iter2,TagName.EXIF_CONTRAST              ,exifSect.GetTagValue(TagName.EXIF_CONTRAST));
				exifTS.AppendValues(iter2,TagName.EXIF_SATURATION            ,exifSect.GetTagValue(TagName.EXIF_SATURATION));
				exifTS.AppendValues(iter2,TagName.EXIF_SHARPNESS             ,exifSect.GetTagValue(TagName.EXIF_SHARPNESS));
				exifTS.AppendValues(iter2,TagName.EXIF_GAIN_CONTROL          ,exifSect.GetTagValue(TagName.EXIF_GAIN_CONTROL));
				exifTS.AppendValues(iter2,TagName.EXIF_CUSTOM_RENDERED       ,exifSect.GetTagValue(TagName.EXIF_CUSTOM_RENDERED));
				iter2 = exifTS.AppendValues(iter,"Subject info","");
				exifTS.AppendValues(iter2,TagName.EXIF_SUBJECT_DISTANCE      ,exifSect.GetTagValue(TagName.EXIF_SUBJECT_DISTANCE));
				exifTS.AppendValues(iter2,TagName.EXIF_SUBJECT_LOCATION      ,exifSect.GetTagValue(TagName.EXIF_SUBJECT_LOCATION));
				exifTS.AppendValues(iter2,TagName.EXIF_SUBJECT_AREA          ,exifSect.GetTagValue(TagName.EXIF_SUBJECT_AREA));
				exifTS.AppendValues(iter2,TagName.EXIF_SUBJECT_DISTANCE_RANGE,exifSect.GetTagValue(TagName.EXIF_SUBJECT_DISTANCE_RANGE));
				iter = exifTS.AppendValues("GPS");
				exifTS.AppendValues(iter,TagName.EXIF_GPS_VERSION           ,exifSect.GetTagValue(TagName.EXIF_GPS_VERSION));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_LATITUDE_REF      ,exifSect.GetTagValue(TagName.EXIF_GPS_LATITUDE_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_LATITUDE          ,exifSect.GetTagValue(TagName.EXIF_GPS_LATITUDE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_LONGITUDE_REF     ,exifSect.GetTagValue(TagName.EXIF_GPS_LONGITUDE_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_LONGITUDE         ,exifSect.GetTagValue(TagName.EXIF_GPS_LONGITUDE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_ALTITUDE_REF      ,exifSect.GetTagValue(TagName.EXIF_GPS_ALTITUDE_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_ALTITUDE          ,exifSect.GetTagValue(TagName.EXIF_GPS_ALTITUDE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DATETIME          ,exifSect.GetTagValue(TagName.EXIF_GPS_DATETIME));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_SATELLITES        ,exifSect.GetTagValue(TagName.EXIF_GPS_SATELLITES));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_STATUS            ,exifSect.GetTagValue(TagName.EXIF_GPS_STATUS));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_MEASURE_MODE      ,exifSect.GetTagValue(TagName.EXIF_GPS_MEASURE_MODE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DOP               ,exifSect.GetTagValue(TagName.EXIF_GPS_DOP));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_SPEED_REF         ,exifSect.GetTagValue(TagName.EXIF_GPS_SPEED_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_SPEED             ,exifSect.GetTagValue(TagName.EXIF_GPS_SPEED));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_TRACK_REF         ,exifSect.GetTagValue(TagName.EXIF_GPS_TRACK_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_TRACK             ,exifSect.GetTagValue(TagName.EXIF_GPS_TRACK));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_IMG_DIRECTION_REF ,exifSect.GetTagValue(TagName.EXIF_GPS_IMG_DIRECTION_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_IMG_DIRECTION     ,exifSect.GetTagValue(TagName.EXIF_GPS_IMG_DIRECTION));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_MAP_DATUM         ,exifSect.GetTagValue(TagName.EXIF_GPS_MAP_DATUM));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_LATITUDE     ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_LATITUDE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_LONGITUDE    ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_LONGITUDE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_BEARING_REF  ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_BEARING_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_BEARING      ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_BEARING));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_DISTANCE_REF ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_DISTANCE_REF));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DEST_DISTANCE     ,exifSect.GetTagValue(TagName.EXIF_GPS_DEST_DISTANCE));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_PROCESSING_METHOD ,exifSect.GetTagValue(TagName.EXIF_GPS_PROCESSING_METHOD));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_AREA_INFORMATION  ,exifSect.GetTagValue(TagName.EXIF_GPS_AREA_INFORMATION));
				exifTS.AppendValues(iter,TagName.EXIF_GPS_DIFFERENTIAL      ,exifSect.GetTagValue(TagName.EXIF_GPS_DIFFERENTIAL));
				
				
				exifTreeView.Model=exifTS;
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.ShowMetadata");
		}
		
		public void SaveMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.SaveMetadata");
			int i=0;
			foreach(object[] image in photoStore)
			{
				if(((PhotoInfo)image[0]).MetadataChanged)
				{
					((PhotoInfo)image[0]).WriteMetadata();
				}
				i++;
				while(Gtk.Application.EventsPending())
				{
					Gtk.Application.RunIteration();
				}
				LoadSaveStatusUpdated(i,numOfPhotosInDirectory,"Saving metadata to files...");
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.SaveMetadata");
		}
		
		public void DiscardMetadataChanges()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.DiscardChanges");
			foreach(object[] photo in photoStore)
			{
				if(((PhotoInfo)photo[0]).MetadataChanged)
				{
					((PhotoInfo)photo[0]).DiscardMetadataChanges();
				}
			}
			GatherMetadata();
			//Console.WriteLine("Leaving Sagittarius.MainWindow.DiscardChanges");
		}
		
		private void GatherMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.GatherMetadata");
			if (selectedPhotos.Count==1)
			{
				metadata = ((PhotoInfo)selectedPhotos[0]).Metadata;
			}
			else
			{
				XmpMetadata newMetadata = new XmpMetadata();
				
				foreach(Sagittarius.PhotoInfo pInfo in selectedPhotos)
				{
					newMetadata.Add(pInfo.Metadata);
				}
				metadata = newMetadata;
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.GatherMetadata");
		}

		private void SetSaveDiscardBtnsStatus(bool sensitive)
		{
			toolbarSaveBtn.Sensitive=sensitive;
			toolbarDiscardBtn.Sensitive=sensitive;
		}
		
		private void AskForSave()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.AskForSave");
			if (CheckUnsavedChanges())
			{
				MessageDialog md = new MessageDialog(
					this,DialogFlags.Modal,
					MessageType.Warning,
					ButtonsType.YesNo,
					"There are changes not saved yet! Do you want to save them?");
				ResponseType mdResult = (ResponseType)md.Run();
				if(mdResult == ResponseType.Yes)
				{
					SaveMetadata();
					SetSaveDiscardBtnsStatus(false);
				}
				else
				{
					DiscardMetadataChanges();
					SetSaveDiscardBtnsStatus(false);
				}
				md.Hide();
				md.Destroy();
			}
			//Console.WriteLine("Entering Sagittarius.MainWindow.AskForSave");
		}

		private bool CheckUnsavedChanges()
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.CheckUnsavedChanges");
			bool result=false;
			if(photoStore!=null)
			{
				foreach(object[] photo in photoStore)
				{
					if(((PhotoInfo)photo[0]).MetadataChanged)
					{
						result = true;
					}
				}
			}
			//Console.WriteLine("Leaving Sagittarius.MainWindow.CheckUnsavedChanges");
			return result;
		}

#endregion

#region simple ui methods
		
		protected virtual void OnDeleteEvent (object o, Gtk.DeleteEventArgs args)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.OnDeleteEvent");
			Quit();
		}
		

		protected virtual void OnMenuFileQuitActivated (object sender, System.EventArgs e)
		{
			menubar1.GrabFocus();
			Quit();
		}

		protected virtual void OnMenuEditPreferencesActivated (object sender, System.EventArgs e)
		{
			menubar1.GrabFocus();
			PreferencesDialog preferencesDialog = new PreferencesDialog();
			preferencesDialog.ShowAll();
			preferencesDialog.Run();
			preferencesDialog.Hide();
			preferencesDialog.Destroy();
		}
		
		protected virtual void OnAboutActivated (object sender, System.EventArgs e)
		{
			menubar1.GrabFocus();
			AboutBox aboutBox = new AboutBox();
			aboutBox.Show();
		}

		protected virtual void OnToolbarSaveBtnActivated (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.OnToolbarSaveBtnActivated");
			toolbar.GrabFocus();
			SaveMetadata();
			SetSaveDiscardBtnsStatus(false);
			//Console.WriteLine("Leaving Sagittarius.MainWindow.OnToolbarSaveBtnActivated");
		}

		protected virtual void OnToolbarDiscardBtnActivated (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering Sagittarius.MainWindow.OnToolbarDiscardBtnActivated");
			toolbar.GrabFocus();
			DiscardMetadataChanges();
			SetSaveDiscardBtnsStatus(false);
			ShowMetadata();
			//Console.WriteLine("Leaving Sagittarius.MainWindow.OnToolbarDiscardBtnActivated");
		}

#endregion

#region iptc-panel
		
#region Creator

		protected virtual void OnCreCreEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CREATOR,creCreEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreJobEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CREATOR_JOB_TITLE,creJobEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreAddTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_ADR_EXTADR,creAddText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreCitEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_ADR_CITY,creCitEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreStaEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_ADR_REGION,creStaEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCrePosEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_ADR_PCODE,crePosEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreCouEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_ADR_CTRY,creCouEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreTelTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_TEL_WORK,creTelText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreEmaTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_EMAIL_WORK,creEmaText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCreWebTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CI_URL_WORK,creWebText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

#endregion
		
#region content

		protected virtual void OnConHeaTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_HEADLINE,conHeaText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnConCapTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_DESCRIPTION,conCapText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnConIptTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_SUBJECT_CODE,conIptText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnConWriEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_DESCRIPTION_WRITER,conWriEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnConKeyTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_KEYWORDS,conKeyText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

#endregion
		
#region image

		protected virtual void OnImaDatPicChanged (object sender, System.EventArgs args)
		{
			if(metadata!=null)
			{
				metadata.IptcSect.GetTag(TagName.IPTC_DATE_CREATED).TagValue=imaDatPic.CurrentDateTime;
				SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaIntEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_INTELLECTUAL_GENRE,imaIntEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaIptTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_SCENE,imaIptText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaLocEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_LOCATION,imaLocEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaCitEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_CITY,imaCitEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaStaEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_PROVINCE_STATE,imaStaEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaCouEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_COUNTRY,imaCouEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnImaIsoEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_COUNTRY_CODE,imaIsoEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

#endregion
		
#region status section
		
		protected virtual void OnStaTitEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_TITLE,staTitEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnStaJobEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_JOB_ID,staJobEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnStaInsTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_INSTRUCTIONS,staInsText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnStaProEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_PROVIDER,staProEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnStaSouEntFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_SOURCE,staSouEnt.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

#endregion
		
#region Copyright
		
		protected virtual void OnCopStaComChanged (object sender, System.EventArgs e)
		{
			if(metadata!=null)
			{
				metadata.IptcSect.SetTagValue(TagName.IPTC_COPYRIGHTED,copStaCom.Active);
				SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCopCopTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_COPYRIGHT_NOTICE,copCopText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

		protected virtual void OnCopRigTextFocusOutEvent (object o, Gtk.FocusOutEventArgs args)
		{
			if(metadata!=null)
			{
			metadata.IptcSect.SetTagValueFromString(TagName.IPTC_RIGHT_USAGE_TERMS,copRigText.Buffer.Text);
			SetSaveDiscardBtnsStatus(CheckUnsavedChanges());
			}
		}

#endregion
		
#endregion
		
	}
}